@@ -4,7 +4,7 @@
  * This file is part of mpop, a POP3 client.
  *
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2014,
- * 2015
+ * 2015, 2016
  * Martin Lambers <marlam@marlam.de>
  * Dimitrios Apostolou <jimis@gmx.net> (UID handling)
  *
@@ -926,6 +926,7 @@ int pop3_capa(pop3_session_t *session, char **errstr)
 int pop3_tls_init(pop3_session_t *session,
         const char *tls_key_file, const char *tls_cert_file,
         const char *tls_trust_file, const char *tls_crl_file,
+        const unsigned char *tls_sha256_fingerprint,
         const unsigned char *tls_sha1_fingerprint,
         const unsigned char *tls_md5_fingerprint,
         int min_dh_prime_bits, const char *priorities,
@@ -933,7 +934,7 @@ int pop3_tls_init(pop3_session_t *session,
 {
     return tls_init(&session->tls, tls_key_file, tls_cert_file,
             tls_trust_file, tls_crl_file,
-            tls_sha1_fingerprint, tls_md5_fingerprint,
+            tls_sha256_fingerprint, tls_sha1_fingerprint, tls_md5_fingerprint,
             min_dh_prime_bits, priorities, errstr);
 }
 #endif /* HAVE_TLS */
